using System;
using UnityEngine;
using UnityEngine.Experimental.Rendering;
using UnityEngine.Events;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Linq;
using MVR;
using MeshVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using GPUTools.Cloth.Scripts;
using Unity.Collections;
using AssetBundles;
using System.Diagnostics;
using Leap;

namespace Stopper {
	public class AdditionalControllers : MVRScript {
		bool isSessionPlugin;
		
		// Prefabricated UI for joint controllers. Not sure how to get ahold of this
		// other than copying from somewhere
		GameObject controllerUIPrefab;
		
		// A sample controller for retrieving controller meshes. Again, not sure
		// how to find this other than copying from elsewhere
		FreeControllerV3 sampleController;
		
		Atom mainAtom;
		
		GameObject jointControl;
		GameObject ui;
		
		GameObject uiPlaceholder;
		
		bool addingToAtom = false;
		List<FreeControllerV3> createdControllers = new List<FreeControllerV3>();
		List<UnityEngine.Object> createdObjects = new List<UnityEngine.Object>();
		
		// Rigidbodies to add controllers for. 
		
		
		// Right Hand
		ControllerConfig rThumb1 = new ControllerConfig("rThumb1", false, "Right Hand");
		ControllerConfig rThumb2 = new ControllerConfig("rThumb2", false, "Right Hand");
		ControllerConfig rThumb3 = new ControllerConfig("rThumb3", true, "Right Hand");
		
		ControllerConfig rIndex1 = new ControllerConfig("rIndex1", false, "Right Hand");
		ControllerConfig rIndex2 = new ControllerConfig("rIndex2", false, "Right Hand");
		ControllerConfig rIndex3 = new ControllerConfig("rIndex3", true, "Right Hand");
		
		ControllerConfig rMid1 = new ControllerConfig("rMid1", false, "Right Hand");
		ControllerConfig rMid2 = new ControllerConfig("rMid2", false, "Right Hand");
		ControllerConfig rMid3 = new ControllerConfig("rMid3", true, "Right Hand");
		
		ControllerConfig rRing1 = new ControllerConfig("rRing1", false, "Right Hand");
		ControllerConfig rRing2 = new ControllerConfig("rRing2", false, "Right Hand");
		ControllerConfig rRing3 = new ControllerConfig("rRing3", true, "Right Hand");
		
		ControllerConfig rPinky1 = new ControllerConfig("rPinky1", false, "Right Hand");
		ControllerConfig rPinky2 = new ControllerConfig("rPinky2", false, "Right Hand");
		ControllerConfig rPinky3 = new ControllerConfig("rPinky3", true, "Right Hand");
		
		ControllerConfig rCarpal1 = new ControllerConfig("rCarpal1", false, "Right Hand");
		ControllerConfig rCarpal2 = new ControllerConfig("rCarpal2", false, "Right Hand");
		
		// Left Hand
		ControllerConfig lThumb1 = new ControllerConfig("lThumb1", false, "Left Hand");
		ControllerConfig lThumb2 = new ControllerConfig("lThumb2", false, "Left Hand");
		ControllerConfig lThumb3 = new ControllerConfig("lThumb3", true, "Left Hand");
		
		ControllerConfig lIndex1 = new ControllerConfig("lIndex1", false, "Left Hand");
		ControllerConfig lIndex2 = new ControllerConfig("lIndex2", false, "Left Hand");
		ControllerConfig lIndex3 = new ControllerConfig("lIndex3", true, "Left Hand");
		
		ControllerConfig lMid1 = new ControllerConfig("lMid1", false, "Left Hand");
		ControllerConfig lMid2 = new ControllerConfig("lMid2", false, "Left Hand");
		ControllerConfig lMid3 = new ControllerConfig("lMid3", true, "Left Hand");
		
		ControllerConfig lRing1 = new ControllerConfig("lRing1", false, "Left Hand");
		ControllerConfig lRing2 = new ControllerConfig("lRing2", false, "Left Hand");
		ControllerConfig lRing3 = new ControllerConfig("lRing3", true, "Left Hand");
		
		ControllerConfig lPinky1 = new ControllerConfig("lPinky1", false, "Left Hand");
		ControllerConfig lPinky2 = new ControllerConfig("lPinky2", false, "Left Hand");
		ControllerConfig lPinky3 = new ControllerConfig("lPinky3", true, "Left Hand");
		
		ControllerConfig lCarpal1 = new ControllerConfig("lCarpal1", false, "Left Hand");
		ControllerConfig lCarpal2 = new ControllerConfig("lCarpal2", false, "Left Hand");
		
		// Head
		ControllerConfig tongue1 = new ControllerConfig("tongue01", false, "Head");
		ControllerConfig tongue2 = new ControllerConfig("tongue02", false, "Head");
		ControllerConfig tongue3 = new ControllerConfig("tongue03", false, "Head");
		ControllerConfig tongue4 = new ControllerConfig("tongue04", false, "Head");
		ControllerConfig tongue5 = new ControllerConfig("tongue05", false, "Head");
		ControllerConfig tongueTip = new ControllerConfig("tongueTip", true, "Head");
		
		ControllerConfig lowerJaw = new ControllerConfig("lowerJaw", false, "Head");
		
		
		// Left Toes
		ControllerConfig lBigToe = new ControllerConfig("lBigToe", false, "Left Toes");
		ControllerConfig lSmallToe1 = new ControllerConfig("lSmallToe1", false, "Left Toes");
		ControllerConfig lSmallToe2 = new ControllerConfig("lSmallToe2", false, "Left Toes");
		ControllerConfig lSmallToe3 = new ControllerConfig("lSmallToe3", false, "Left Toes");
		ControllerConfig lSmallToe4 = new ControllerConfig("lSmallToe4", false, "Left Toes");
		
		// Right Toes
		ControllerConfig rBigToe = new ControllerConfig("rBigToe", false, "Right Toes");
		ControllerConfig rSmallToe1 = new ControllerConfig("rSmallToe1", false, "Right Toes");
		ControllerConfig rSmallToe2 = new ControllerConfig("rSmallToe2", false, "Right Toes");
		ControllerConfig rSmallToe3 = new ControllerConfig("rSmallToe3", false, "Right Toes");
		ControllerConfig rSmallToe4 = new ControllerConfig("rSmallToe4", false, "Right Toes");
		
		
		// Glutes
		ControllerConfig RGlute = new ControllerConfig("RGlute", false, "Glutes");
		ControllerConfig LGlute = new ControllerConfig("LGlute", false, "Glutes");
		
			// Glutes
		ControllerConfig lPectoral = new ControllerConfig("lPectoral", false, "Pectorals");
		ControllerConfig rPectoral = new ControllerConfig("rPectoral", false, "Pectorals");
		
		
		// Other
		ControllerConfig _JointGl = new ControllerConfig("_JointGl", false, "Gens", "VaginaL");
		ControllerConfig _JointGr = new ControllerConfig("_JointGr", false, "Gens","VaginaR");
		
		ControllerConfig _JointAl = new ControllerConfig("_JointAl", false, "Gens","AnusL");
		ControllerConfig _JointAr = new ControllerConfig("_JointAr", false, "Gens","AnusR");
		
		ControllerConfig _JointB = new ControllerConfig("_JointB", false, "Gens", "Base");
		
		
		//ControllerConfig gen1 = new ControllerConfig("Gen1", false);
				
		// Plugin UI
		JSONStorableFloat newControllerSelectRadius;
		JSONStorableFloat handSelectRadius;
		JSONStorableFloat allSelectRadius;
		
		JSONStorableFloat controllerMeshScale;
		
		public override void Init() {
			// Might be a better way to do this, but works for now
			isSessionPlugin = containingAtom.name == "CoreControl";
			
			if(isSessionPlugin){
				CreateTextField(new JSONStorableString("sessionUIWarning", "UI settings do not work at the session level.\n\nTo use them, add this plugin to the model as well."), false);
			} else {
				CreateUI();
			}
		}
		
		private void CreateUI(){
			newControllerSelectRadius = AddSlider("Added Controller Select Radius", 0.01f, 0.01f, 0.1f, delegate {SyncControllerUISettings();});
			handSelectRadius = AddSlider("Hand Select Radius", 0.03f, 0.01f, 0.1f, delegate {SyncControllerUISettings();});
			allSelectRadius = AddSlider("All Controller Select Radius", 0.1f, 0.01f, 0.1f, delegate {SyncControllerUISettings();});
			
			controllerMeshScale = AddSlider("Controller Mesh Scale", 0.2f, 0.1f, 0.5f, delegate {SyncControllerUISettings();});
		}
		
		private void SyncControllerUISettings(){
			if(jointControl != null){
				foreach(Transform childTransform in jointControl.transform){
					GameObject control = childTransform.gameObject;
					
					float selectRadius = allSelectRadius.val;
					
					if(control.name.Contains("Hand")){
						selectRadius = handSelectRadius.val;
					}
					
					foreach(ControllerConfig config in ControllerConfig.All){
						if(control.name.Contains(config.name)){
							selectRadius = newControllerSelectRadius.val;
							break;
						}
					}
					
					control.GetComponent<SphereCollider>().radius = selectRadius;
					
				}
			}
			
			foreach(ControllerConfig config in ControllerConfig.All){
				config.freeController.meshScale = controllerMeshScale.val;
			}
		}
		
		private void SyncControllerHidden(){
			foreach(ControllerConfig config in ControllerConfig.All){
				if(!config.isVisible.val){
					config.SetVisible(false);
				}
			}
		}
		
		private GameObject FindGameObject(GameObject parent, string name){
			Transform childTransform = parent.transform.Find(name);
			
			if(childTransform == null){
				return null;
			} else {
				return childTransform.gameObject;
			}
		}
		
		private GameObject CreateChildObject(GameObject parent, string name) {
			var gameObject = new GameObject(name);
			gameObject.transform.parent = parent.transform;

			createdObjects.Add(gameObject);
			
			return gameObject;
		}
		
		private Component CreateComponent(GameObject parent, System.Type type) {
			var component = parent.AddComponent(type) as Component;
			createdObjects.Add(component);
			return component;
		}
		
		List<GameObject> GetAllChildren(GameObject parent, List<GameObject> objectList = null)
		{
			if (objectList == null) objectList = new List<GameObject>();
		  
			foreach (Transform child in parent.transform) {
				objectList.Add(child.gameObject);
				GetAllChildren(child.gameObject, objectList);
			}
			return objectList;
		}
		
		//-------Start------
		private void Start()
		{
			StartCoroutine("AddControllers");
		}
		
		private void Update()
		{
			if(!isSessionPlugin) {
				SyncControllerHidden();
			}
		}
		
		private IEnumerator AddControllers()
		{
			yield return new WaitForSeconds(0.01f);

			/* 
				When added as a session plugin, this script works by modifying the Person prefab - the template from which other People are created.
				
				Why? Because Atoms have an immutable freeControllers array that's used for various things like 'select
				collider from scene'. 
				
				If that doesn't matter to you, set mainAtom = containingAtom and delete all the prefab code.			
			*/
			if(isSessionPlugin){
				
				// Check to see if the prefab already exists (there's a person in the scene)
				GameObject personPrefab = SuperController.singleton.GetCachedPrefab("a_per", "Person");
				if(personPrefab == null){
					// No people in the scene yet. Load one (without adding to scene) and register it as the prefab. 
					// This might have unintended consequences. 
					SuperController.LogMessage("Person prefab not cached. Loading from asset bundle.");
					AssetBundleLoadAssetOperation request = AssetBundleManager.LoadAssetAsync("a_per", "Person", typeof(GameObject));
					if (request == null)
					{
						SuperController.LogError("Unable to load Person prefab");
						yield break;
					}
					yield return base.StartCoroutine(request);

					personPrefab = request.GetAsset<GameObject>();
					if (personPrefab != null)
					{
						SuperController.singleton.RegisterPrefab("a_per", "Person", personPrefab);
						mainAtom = personPrefab.GetComponent<Atom>();
					} else {
						SuperController.LogError("Unable to load Person prefab");
						yield break;
					}
				} else {
					mainAtom = personPrefab.GetComponent<Atom>();
				}
			} else {
				// If we didn't add it as a session atom, just apply it to the current atom.
				addingToAtom = true;
				mainAtom = containingAtom;
			}
			
			GameObject sampleHipControl = null;
			
			// Load an existing controller joint ui to use as a template. Not sure how else to do it.
			foreach (GameObject go in Resources.FindObjectsOfTypeAll(typeof(GameObject)) as GameObject[]){
				if(go.name =="UIFreeControllerJoint") {
					controllerUIPrefab = go;
					continue;
				}
			}
			
			foreach (GameObject go in GetAllChildren(mainAtom.gameObject)){
				if(go.name == "hipControl") {
					sampleHipControl = go;
				} else if (go.name == "JointControl") {
					jointControl = go;
				} else if (go.name == "UI" && go.transform.parent.gameObject == mainAtom.gameObject) {
					ui = go;
				}
				
				//if(go.name == "Gen1") {
				//	go.active = true;
				//}
			}
			
			sampleController = sampleHipControl.GetComponent<FreeControllerV3>();
			
				
			
			foreach(Rigidbody rb in mainAtom.gameObject.GetComponentsInChildren<Rigidbody>()){
				ControllerConfig config;
				if(rb.name.ToLower() == "pelvis"){
					foreach(Rigidbody test in rb.gameObject.GetComponentsInChildren<Rigidbody>()){
					//	SuperController.LogMessage($"test: {test.name}");
					}
				}
				if(ControllerConfig.ByName.TryGetValue(rb.name, out config)){
					config.controlledRB = rb;
				}
			}
			
			string group = "";
			foreach(ControllerConfig controllerConfig in ControllerConfig.All){
				Rigidbody rb = controllerConfig.controlledRB;
				
				if(rb == null) {
					SuperController.LogMessage($"Could not find rigid body " + controllerConfig.name + ". Skipping.");
					continue;
				}
				
				rb.solverIterations = 10;
				
				bool previouslyLoaded = FindGameObject(jointControl, rb.name+"Control") != null;
				if(!previouslyLoaded){
				
					// Parent object for controls
					//  Needs a Rigidbody, SphereCollider, FreeControllerV3, and a joint tying it to the target Rigidbody
					GameObject control = CreateChildObject(jointControl, rb.name+"Control");
					controllerConfig.controlGameObject = control;
					
					control.SetActive(false);
					
					control.layer = 8;
					
					ForceReceiver forceReceiver = CreateComponent(rb.gameObject, typeof(ForceReceiver)) as ForceReceiver;
					forceReceiver.containingAtom = mainAtom;

					Rigidbody controlRB = CreateComponent(control, typeof(Rigidbody)) as Rigidbody;
					controlRB.isKinematic = true;
					controlRB.useGravity = false;
					controlRB.detectCollisions = true;

					ConfigurableJoint controlJoint = null;

					foreach(ConfigurableJoint joint in rb.gameObject.GetComponents<ConfigurableJoint>()){
						if(joint.connectedBody == controlRB){
							controlJoint = joint;
						}
					}
					
					if(controlJoint == null){
						controlJoint = CreateComponent(rb.gameObject, typeof(ConfigurableJoint)) as ConfigurableJoint; 
					}
					
					controlJoint.connectedBody = controlRB;
					controlJoint.autoConfigureConnectedAnchor = false;
					controlJoint.anchor = Vector3.zero;
					controlJoint.connectedAnchor = Vector3.zero;
					
					controllerConfig.controlJoint = controlJoint;
					
					FreeControllerV3 controller = CreateComponent(control, typeof(FreeControllerV3)) as FreeControllerV3;
					controllerConfig.freeController = controller;
					controller.containingAtom = mainAtom;
					controller.followWhenOff = rb.transform;
					controller.useGravityOnRBWhenOff = false;
					controller.meshScale = 0.3f;
					controller.selectedScale = 0.1f;
					controller.enableSelectRoot = true;
					controller.enabled = true;
					controller.hidden = false;
					controller.startingPositionState = FreeControllerV3.PositionState.Off;
					controller.startingRotationState = FreeControllerV3.RotationState.Off;
					
					createdControllers.Add(controller);
					
					// Add it to the SuperController's free controller list so that global enable/disable works
					if(addingToAtom && !SuperController.singleton.GetAllFreeControllers().Contains(controller)){
						SuperController.singleton.GetAllFreeControllers().Add(controller);
					}
					
					controller.material = sampleController.material;
					if(controller.onPositionMesh == null){
						controller.onPositionMesh = sampleController.onPositionMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.onPositionMesh, controller.transform);
						controller.offPositionMesh = sampleController.offPositionMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.offPositionMesh, controller.transform);
						controller.followingPositionMesh = sampleController.followingPositionMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.followingPositionMesh, controller.transform);
						controller.holdPositionMesh = sampleController.holdPositionMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.holdPositionMesh, controller.transform);
						controller.lockPositionMesh = sampleController.lockPositionMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.lockPositionMesh, controller.transform);
						controller.onRotationMesh = sampleController.onRotationMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.onRotationMesh, controller.transform);
						controller.offRotationMesh = sampleController.offRotationMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.offRotationMesh, controller.transform);
						controller.followingRotationMesh = sampleController.followingRotationMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.followingRotationMesh, controller.transform);
						controller.holdRotationMesh = sampleController.holdRotationMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.holdRotationMesh, controller.transform);
						
						controller.lockRotationMesh = sampleController.lockRotationMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.lockRotationMesh, controller.transform);
						controller.lookAtRotationMesh = sampleController.lookAtRotationMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.lookAtRotationMesh, controller.transform);
						controller.deselectedMesh = sampleController.deselectedMesh == null ? null :  UnityEngine.Object.Instantiate<Mesh>(sampleController.deselectedMesh, controller.transform);;
					}
					SphereCollider collider = CreateComponent(control, typeof(SphereCollider)) as SphereCollider;
					collider.enabled = true;
					collider.radius = 0.01f;
					
					// Controller Link. Not sure what this is used for.
					GameObject controlLink = CreateChildObject(jointControl, rb.name+"Link");
					controllerConfig.linkGameObject = controlLink;
					controlLink.layer = 8;
					
					SphereCollider linkCollider = CreateComponent(controlLink, typeof(SphereCollider)) as SphereCollider;
					linkCollider.enabled = true;
					linkCollider.radius = 0.01f;
					
					Rigidbody linkRB = CreateComponent(controlLink, typeof(Rigidbody)) as Rigidbody;
					linkRB.isKinematic = true;
					linkRB.useGravity = false;
					linkRB.detectCollisions = false;
					
					FreeControllerV3Link freeControllerLink = CreateComponent(controlLink, typeof(FreeControllerV3Link)) as FreeControllerV3Link;
					freeControllerLink.linkedController = controller;
					
					// UI			
					uiPlaceholder = CreateChildObject(ui, "UIPlaceholder"+rb.name );
					uiPlaceholder.SetActive(false);
					
					
					MoveAndRotateAsHUDAnchor moveAsHud = CreateComponent(uiPlaceholder, typeof(MoveAndRotateAsHUDAnchor)) as MoveAndRotateAsHUDAnchor;
					moveAsHud.scaleAsAnchor = true;
					moveAsHud.anchorNum = HUDAnchor.AnchorNum.Two;
										
					UIConnectorMaster connectorMaster = CreateComponent(uiPlaceholder, typeof(UIConnectorMaster)) as UIConnectorMaster;
					connectorMaster.UIPrefab = controllerUIPrefab.transform;
							
					UIConnector connector = CreateComponent(uiPlaceholder, typeof(UIConnector)) as UIConnector;
					connector.receiver = controller;
					connector.receiverTransform = control.transform;
					
					
					controller.UITransforms = new Transform[]{uiPlaceholder.transform};
					controller.UITransformsPlayMode = new Transform[]{};
					

					control.SetActive(true);
			

					// Reconnect the joint otherwise things break
					controlJoint.connectedBody = null;
					controlJoint.connectedBody = controlRB;
					

					// Needs to be done here or else things break. not sure why
					controller.guihidden = true;

				} else {
					GameObject control = FindGameObject(jointControl, rb.name+"Control");
					controllerConfig.controlGameObject = control;
					
					FreeControllerV3 controller = control.GetComponentInChildren<FreeControllerV3>(true);
					controllerConfig.freeController = controller;
					
					GameObject controlLink = FindGameObject(jointControl, rb.name+"Link");
					controllerConfig.linkGameObject = controlLink;
				}
				
				
				if(!isSessionPlugin){
					if(controllerConfig.group != group){
						group = controllerConfig.group;
						CreateTextField(new JSONStorableString(group, group), false).height = 10;
					}
					controllerConfig.SetVisible(controllerConfig.defaultVisible);
					controllerConfig.isVisible = AddToggle(controllerConfig.displayName + " Controller Visible", controllerConfig.defaultVisible, (isVisible) => {
						controllerConfig.SetVisible(isVisible);
					});
				}
			}
			
			// Add the new controllers to the AllJointsController so they work with the Key/All joint controls
			AllJointsController allJointsController = jointControl.GetComponent<AllJointsController>();
			List<FreeControllerV3> freeControllers = new List<FreeControllerV3>();
			freeControllers.AddRange(allJointsController.freeControllers);
			
			foreach(FreeControllerV3 newController in createdControllers){
				if(!freeControllers.Contains(newController)){
					freeControllers.Add(newController);
				}
			}
			allJointsController.freeControllers = freeControllers.ToArray();
			
			SyncControllerUISettings();
		}
		
		private void OnDisable()
		{
			// Clean up maybe? Seems like a hassle.
		}

		private void OnDestroy(){
			foreach(var o in createdObjects){
				Destroy(o);
			}
		}
		
		//UI Functions
		
		public JSONStorableFloat AddSlider(string name, float defaultVal, float min, float max, UnityAction<float> onChange = null, bool rightSide = false)
		{
			JSONStorableFloat storableFloat = new JSONStorableFloat(name, defaultVal, min, max, true, true);
			storableFloat.storeType = JSONStorableParam.StoreType.Full;
			UIDynamicSlider uiSlider = this.CreateSlider(storableFloat, rightSide);
			RegisterFloat(storableFloat);
			
			if(onChange != null){
				uiSlider.slider.onValueChanged.AddListener(onChange);
			}
			
			return storableFloat;
		}
		
		public JSONStorableBool AddToggle(string label, bool defaultVal, UnityAction<bool> onChange = null, bool rightSide = false)
		{
			JSONStorableBool storableBool = new JSONStorableBool(label, defaultVal);
			UIDynamicToggle uiToggle = this.CreateToggle(storableBool, rightSide);
			RegisterBool(storableBool);
			
			if(onChange != null){
				uiToggle.toggle.onValueChanged.AddListener(onChange);
			}
			
			return storableBool;
		}
		
		private class ControllerConfig {
			public string name;
			public string group;
			public string displayName;
			public bool defaultVisible;
			
			public Rigidbody controlledRB;
			public FreeControllerV3 freeController;
			public ConfigurableJoint controlJoint;
			
			public GameObject controlGameObject;
			public GameObject linkGameObject;
			
			// UI Elements
			public JSONStorableBool isVisible;
			
			public static Dictionary<string, ControllerConfig> ByName = new Dictionary<string, ControllerConfig>();
			public static List<ControllerConfig> All = new List<ControllerConfig>();
			
			public ControllerConfig(string name, bool defaultVisible, string group, string displayName = null){
				this.name = name;
				this.defaultVisible = defaultVisible;
				this.group = group;
				
				if(displayName != null){
					this.displayName = displayName;
				} else {
					this.displayName = name;
				}
				
				ByName[name] = this;
				All.Add(this);
			}
			
			public void SetVisible(bool visible){
				if(visible){
					freeController.hidden = false;
				
					controlGameObject.layer = 8;
					linkGameObject.layer = 8;
				} else {
					freeController.hidden = true;
					
					controlGameObject.layer = Physics.IgnoreRaycastLayer;
					linkGameObject.layer = Physics.IgnoreRaycastLayer;
				}
			}
		}
	}
}